package org.rcisoft.sys.attendance.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.attendance.entity.Attendance;
import org.rcisoft.sys.attendance.service.AttendanceService;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.sysuser.dao.DoorRecordRepository;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.entity.DoorRecord;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class AttendanceServiceImpl implements AttendanceService {

    @Autowired
    DoorRecordRepository doorRecordRepository;

    @Autowired
    private SysDictDataRepositorys sysDictDataRepositorys;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Override
    public List<Attendance> findAll(Attendance attendance) {
        List<Attendance> attendanceList = new ArrayList<>();
        // 根据时间范围筛选所有打卡成功的记录
        // 获取前一天的时间
        // TODO: 2023/1/1  临时修改获取当天的数据
        Date update = TimeUtils.getUpdate();
        attendance.setBeginTime(TimeUtils.getDateNotTime(update));
        attendance.setEndTime(TimeUtils.getDateEndDay(update));
        // 查询时间范围内员工的正常记录集合
        List<DoorRecord> doorRecordList = doorRecordRepository.queryAttendance(attendance);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//这里可以指定日期的格式
        // todo 2022/12/30 根据用户分组(每个员工时间范围内的所有打卡记录)
        Map<String, List<DoorRecord>> listMap = doorRecordList.stream()
                .collect(Collectors.groupingBy(DoorRecord::getPersonId));
        for (String key : listMap.keySet()) {
            List<DoorRecord> records = listMap.get(key);
            // 某个员工某日所有通行记录
            Attendance att = new Attendance();
            att.setPeopleName(records.get(0).getName());
            att.setPhoneNumber(records.get(0).getMobile());
            att.setDeptName(records.get(0).getDeptName());
            att.setPositionName(records.get(0).getPositionName());
            att.setNltBz(records.get(0).getNltBz());
            att.setKqDate(sdf.format(update));
            att.setUserId(records.get(0).getUserId());
            att = setValue(records, att);
            if (att != null)
                attendanceList.add(att);
        }
        return attendanceList;
    }

    // 整合考勤信息
    Attendance setValue(List<DoorRecord> records, Attendance attendance) {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        SimpleDateFormat time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        attendance.setDuration(BigDecimal.ZERO);
        // 根据时间升序排序
        records = records.stream().sorted(Comparator.comparing(DoorRecord::getCrossTime)).collect(Collectors.toList());
        // 获取第一个进入时间为上班时间
        DoorRecord work = records.stream().filter(r -> r.getDeviceType().equals("1")).findFirst().orElse(null);
        //降序排序
        records = records.stream().sorted(Comparator.comparing(DoorRecord::getCrossTime).reversed()).collect(Collectors.toList());
        // 获取最后的出去时间
        DoorRecord out = records.stream().filter(r -> r.getDeviceType().equals("2")).findFirst().orElse(null);
        // 判断最后出去的时间是否大于最早进来的时间
        if (work != null && out != null) {
            String begin = time.format(work.getCrossTime());
            String end = time.format(out.getCrossTime());
            if (begin.compareTo(end) > 0)
                return null;
            attendance.setWorkShift(sdf.format(work.getCrossTime()));
            attendance.setWorkDeviceName(work.getDeviceName());
            attendance.setClosingTime(sdf.format(out.getCrossTime()));
            attendance.setOfficeDeviceName(out.getDeviceName());
        } else
            return null;
        Date beginTime = work.getCrossTime();
        Date endTime = out.getCrossTime();
        long sumTime = 0l;
/*        for (int i = 0; i < records.size(); i++) {
            if (records.get(i).getDeviceType().equals("1"))
                beginTime = records.get(i).getCrossTime();
            if (records.get(i).getDeviceType().equals("2"))
                endTime = records.get(i).getCrossTime();
            if (beginTime != null && endTime != null && beginTime.compareTo(endTime) < 0) {
                sumTime += TimeUtils.getDatePoor(beginTime, endTime);
                beginTime = null;
                endTime = null;
            }
        }*/
        sumTime = TimeUtils.getDatePoor(beginTime, endTime);
        if (sumTime > 0) {
            BigDecimal totalFee = new BigDecimal(sumTime);
            BigDecimal d100 = new BigDecimal(60 * 60 * 1000);
            BigDecimal big3 = totalFee.divide(d100, 1, RoundingMode.HALF_UP);
            attendance.setDuration(big3);
        }
        // 查询所有班次
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("shift");
        List<SysDictData> shiftList = sysDictDataRepositorys.querySysDictDatas(sysDictData);
        SysDictData data = shiftList.stream().filter(s -> s.getDictLabel()
                .equals(attendance.getNltBz())).findFirst().orElse(null);
        attendance.setKqFlag("异常");
        // todo 2022/12/30 做data 的非空校验
        if (data != null && data.getRemarks() != null) {
            // 班次最小时长
            BigDecimal xz = new BigDecimal(data.getRemarks());
            if (xz.compareTo(attendance.getDuration())!=1)
                attendance.setKqFlag("正常");
        }
        return attendance;
    }

    public List<DoorRecord> findAllBy(Attendance attendance, HttpServletRequest request) {
        //从token获取用户id
        String userId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
        if (userId != null) {
            attendance.setUserId(Integer.valueOf(userId));
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date1 = "2022-01-01 00:00:01";
        String date2 = "2023-12-31 11:59:59";
        try {
            attendance.setBeginTime(simpleDateFormat.parse(date1));
            attendance.setEndTime(simpleDateFormat.parse(date2));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        attendance.setBeginTime(TimeUtils.getDateNotTime(attendance.getBeginTime()));
        attendance.setEndTime(TimeUtils.getDateEndDay(attendance.getEndTime()));
        List<DoorRecord> doorRecordList = doorRecordRepository.queryAttendance2(attendance);
        return doorRecordList;
    }
}
